@@ -97,6 +97,7 @@ G_DEFINE_QUARK (autoar-extractor, autoar_extractor)
 #define BUFFER_SIZE (64 * 1024)
 #define NOT_AN_ARCHIVE_ERRNO 2013
 #define EMPTY_ARCHIVE_ERRNO 2014
+#define WRONG_PASSPHRASE_ERRNO 2015
 
 typedef struct _GFileAndInfo GFileAndInfo;
 
@@ -133,6 +134,8 @@ struct _AutoarExtractor
 
   GList *files_list;
 
+  gchar *passphrase;
+
   GHashTable *userhash;
   GHashTable *grouphash;
   GArray     *extracted_dir_list;
@@ -160,6 +163,7 @@ enum
   SCANNED,
   DECIDE_DESTINATION,
   PROGRESS,
+  REQUEST_PASSPHRASE,
   CONFLICT,
   CANCELLED,
   COMPLETED,
@@ -684,6 +688,39 @@ libarchive_read_skip_cb (struct archive *ar_read,
   return 0;
 }
 
+static inline void
+autoar_extractor_signal_request_passphrase (AutoarExtractor *self)
+{
+  autoar_common_g_signal_emit (self, self->in_thread,
+                               autoar_extractor_signals[REQUEST_PASSPHRASE], 0,
+                               &self->passphrase);
+}
+
+static const char *
+libarchive_read_set_passphrase_cb (struct archive *ar_read,
+                                   void           *client_data)
+{
+  AutoarExtractor *self = AUTOAR_EXTRACTOR (client_data);
+  GError *error = NULL;
+
+  if (self->passphrase == NULL) {
+    autoar_extractor_signal_request_passphrase (self);
+
+    return self->passphrase;
+  }
+
+  error = g_error_new (AUTOAR_EXTRACTOR_ERROR,
+                       WRONG_PASSPHRASE_ERRNO,
+                       "\'%s\': %s",
+                       self->source_basename,
+                       "wrong passphrase");
+  autoar_common_g_signal_emit (self, self->in_thread,
+                               autoar_extractor_signals[AR_ERROR], 0,
+                               error);
+
+  return NULL;
+}
+
 static int
 libarchive_create_read_object (gboolean          use_raw_format,
                                AutoarExtractor  *self,
@@ -701,6 +738,7 @@ libarchive_create_read_object (gboolean          use_raw_format,
   archive_read_set_seek_callback (*a, libarchive_read_seek_cb);
   archive_read_set_skip_callback (*a, libarchive_read_skip_cb);
   archive_read_set_callback_data (*a, self);
+  archive_read_set_passphrase_callback (*a, self, libarchive_read_set_passphrase_cb);
 
   return archive_read_open1 (*a);
 }
@@ -1405,6 +1443,15 @@ autoar_extractor_class_init (AutoarExtractorClass *klass)
                   G_TYPE_UINT64,
                   G_TYPE_UINT);
 
+
+  autoar_extractor_signals[REQUEST_PASSPHRASE] =
+    g_signal_new ("request-passphrase",
+                  type,
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_generic,
+                  G_TYPE_POINTER,
+                  0);
 /**
  * AutoarExtractor::conflict:
  * @self: the #AutoarExtractor
@@ -1592,10 +1639,6 @@ autoar_extractor_step_scan_toplevel (AutoarExtractor *self)
       return;
     }
 
-    if (archive_entry_is_encrypted (entry)) {
-      break;
-    }
-
     if (self->use_raw_format) {
       pathname = autoar_common_get_basename_remove_extension (g_file_get_path(self->source_file));
       g_debug ("autoar_extractor_step_scan_toplevel: %d: raw pathname = %s",
@@ -1618,17 +1661,6 @@ autoar_extractor_step_scan_toplevel (AutoarExtractor *self)
     archive_read_data_skip (a);
   }
 
-  if (entry && archive_entry_is_encrypted (entry)) {
-    g_debug ("autoar_extractor_step_scan_toplevel: encrypted entry");
-    if (self->error == NULL) {
-      self->error = g_error_new (G_IO_ERROR,
-                                 G_IO_ERROR_NOT_SUPPORTED,
-                                 "Encrypted archives are not supported.");
-    }
-    archive_read_free (a);
-    return;
-  }
-
   if (self->files_list == NULL) {
     if (self->error == NULL) {
       self->error = g_error_new (AUTOAR_EXTRACTOR_ERROR,
